/**
  *  \file io/zip.h
  *  \brief Zip file access.
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_IO_ZIP_H
#define PCC_V2_IO_ZIP_H

#include "io/dirs.h"
#include "cpluslib/smartptr.h"
#include "cpluslib/ptrvector.h"

/** Zip file access. This provides simple, read-only access to a Zip file.
    It builds an in-memory index of the member files on-the fly.

    Note that ZipFile instances must be managed by Ptr<>, which is
    required for all Directory descendants. Directory entries or enumerations
    will keep the ZipFile instance alive, stream members will keep the
    underlying file alive.

    Limitations:
    - we support only "classic" PKZIP 2.0 compression (deflate), no Deflate64,
      LZMA, etc. This is the most common format, in particular in VGAP.
    - the Zip file is "flattened" i.e. all directory structure removed.
    - compressed members have the same restrictions that ZipStream has:
      only forward reading, no seeking, no rewinding.
    - unlike many zippers, this one reads the local headers, not the central
      headers.
    - unless specifically marked as Unicode, file names are treated as game
      character set. This is valid for files originating on DOS, but otherwise
      probably wrong. FIXME: can we use the operating system filename
      character set? (CP_ACP resp. LC_CTYPE) */
class ZipFile : public Directory {
    Ptr<Stream> file;      /**< Underlying file. */
    ChildStream view;      /**< Isolation against other users. */
    int options;

    // Index
    struct IndexEntry;
    ptr_vector<IndexEntry> index;
    bool indexer_reached_end;
    bool had_unsupported_files;
    bool had_unsupported_feature;
    bool had_duplicate_files;

    // Directory interface
    class ZipDirEntry;
    class ZipDirEnum;
    class ZipStoredMember;
    class ZipDeflatedMember;
    friend class ZipDirEntry;
    friend class ZipDirEnum;
    friend class ZipStoredMember;
    friend class ZipDeflatedMember;

    bool readNextEntry();
    IndexEntry* findEntryByName(const string_t& name, std::size_t start, std::size_t end);

    ZipFile(Ptr<Stream> file, int options);

 public:
    enum {
        KeepPaths = 1
    };
    
    static Ptr<ZipFile> open(Ptr<Stream> file, int options);

    ~ZipFile();

    Ptr<DirectoryEntry> getEntryByName(string_t name);
    Ptr<Enumeration<Ptr<DirectoryEntry> > > getEntries();
    Ptr<Directory> getParent();
    string_t getName();
};

#endif
