/**
  *  \file ui/widgets/focusiterator.cc
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "ui/widgets/focusiterator.h"

/**************************** UIFocusIterator ****************************/

/** \class UIFocusIterator
    \brief Cursor Focus Control

    This invisible widget provides a means of manipulating focus using
    the keyboard. Given a range of widget Ids, it provides keys to
    move within this range.
    - fi_Horiz: left and right arrows
    - fi_Vert: up and down arrows
    - fi_Tab: Tab and Shift-Tab keys
    - fi_Home: accepts Home and End to move to ends
    - fi_Page: accepts PgUp and PgDn to move to ends
    - fi_Wrap: when added to fi_Horiz/fi_Vert, those "wrap around". By
      default, only fi_Tab wraps around. */

UIFocusIterator::UIFocusIterator(int from, int to, int flags)
    : UIFixedWidget(), from(from), to(to), flags(flags)
{
    setState(st_Disabled, true);  // so it doesn't try to get the focus
}

UIFocusIterator::~UIFocusIterator()
{ }

void
UIFocusIterator::drawContent(GfxCanvas& /*can*/)
{ }

bool
UIFocusIterator::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Key) {
        int type = 0;
        switch(e.key.code) {
         case SDLK_TAB:            type = fi_Tab; break;
         case SDLK_TAB + ss_Shift: type = fi_Tab + fi_INTERNAL_NEGATE; break;
         case SDLK_UP:             type = fi_Vert + fi_INTERNAL_NEGATE; break;
         case SDLK_DOWN:           type = fi_Vert; break;
         case SDLK_LEFT:           type = fi_Horiz + fi_INTERNAL_NEGATE; break;
         case SDLK_RIGHT:          type = fi_Horiz; break;
         case SDLK_HOME:
         case SDLK_HOME + ss_Ctrl: type = fi_Home + fi_INTERNAL_NEGATE; break;
         case SDLK_END:
         case SDLK_END + ss_Ctrl:  type = fi_Home; break;
         case SDLK_PAGEUP:         type = fi_Page + fi_INTERNAL_NEGATE; break;
         case SDLK_PAGEDOWN:       type = fi_Page; break;
        }
        if (type & flags) {
            /* Locate a focused widget which we know. Do not go past our parent. */
            UIBaseWidget* focused_w = getParent()->getFocusedWidget();
            while (focused_w != 0
                   && (focused_w->getId() < from || focused_w->getId() > to)
                   && focused_w != getParent())
            {
                focused_w = focused_w->getParent();
            }

            type |= flags & fi_Wrap;

            /* If we found one, we know the focused Id. Find new Id */
            if (focused_w != 0 && focused_w != getParent()) {
                int focus_id = focused_w->getId();
                if (type & (fi_Page | fi_Home)) {
                    /* in Page/Home mode, move backward from end */
                    if (type & fi_INTERNAL_NEGATE) {
                        focus_id = from-1;
                        type = 0;
                    } else {
                        focus_id = to+1;
                        type = fi_INTERNAL_NEGATE;
                    }
                }

                int new_id = focus_id;
                UIBaseWidget* new_widget = 0;
                do {
                    if (type & fi_INTERNAL_NEGATE)
                        --new_id;
                    else
                        ++new_id;
                    if (new_id < from) {
                        if (type & (fi_Tab | fi_Wrap))
                            new_id = to;
                        else
                            return true;
                    } else if (new_id > to) {
                        if (type & (fi_Tab | fi_Wrap))
                            new_id = from;
                        else
                            return true;
                    }
                    new_widget = getParent()->getWidgetById(new_id);
                } while (new_id != focus_id && (new_widget == 0 || new_widget->hasState(st_Disabled)));
                if (new_id != focus_id) {
                    setSelectedWidget();
                    new_widget->setFocusedWidget();
                }
                return true;
            }
        } /* not a focus iterator key */
    } /* not a key */
    return UIFixedWidget::handleEvent(e, second_pass);
}
